package dk.jkkn.util;

import java.net.URL;

/**
 * Utility class to abstract away differences in looking at resources versa
 * files.<br/><br/>
 * 
 * This class wraps a URL inside a simple API to easily determined if we
 * are considering a directory, file and whether it is readable.
 * 
 * @author Kristian Krmmer Nielsen <jkkn@jkkn.dk>
 */
public class ResourceWrap {

	private final URL url;
	private final boolean isDirectory;
	private final boolean isFile;
	private final boolean canRead;

	/**
	 * Creates new instance
	 * 
	 * @param url URL object to represent
	 * @param isDirectory Whether the URL is readable
	 * @param isFile true if URL is representing a file
	 * @param canRead true if URL is readable
	 */
	protected ResourceWrap(final URL url, final boolean isDirectory,
			final boolean isFile, final boolean canRead) {
		super();
		this.url = url;
		this.isDirectory = isDirectory;
		this.isFile = isFile;
		this.canRead = canRead;
	}

	/**
	 * Returns true if URL is a directory
	 * 
	 * @return true if directory
	 */
	public boolean isDirectory() {
		return isDirectory;
	}

	/**
	 * Returns true if URL if a file
	 * 
	 * @return true if file
	 */
	public boolean isFile() {
		return isFile;
	}

	/**
	 * Returns the URL object
	 * 
	 * @return URL
	 */
	public URL getURL() {
		return url;
	}

	/**
	 * Returns true if readable
	 * 
	 * @return true if readable
	 */
	public boolean canRead() {
		return canRead;
	}

	/**
	 * Returns the filename part of the URL
	 * 
	 * @return filename of URL
	 */
	public String getName() {
		return url.getFile();
	}

}
