/**
 * 
 */
package dk.jkkn.util.swixml;

import org.jdom.Attribute;
import org.swixml.Converter;
import org.swixml.Localizer;

/**
 * Converts comma-seperated string into array of strings
 * 
 * @author Kristian Krmmer Nielsen <jkkn@jkkn.dk>
 */
public class StringArrayConverter implements Converter {

	/**
	 * Converts a comma-seperated string into a string array.
	 * 
	 * @param type The type we convert into (String[].array)
	 * @param attr Attribute to convert
	 * @param localizer Localizer we use to localize each part of the array
	 * @see org.swixml.Converter#convert(java.lang.Class, org.jdom.Attribute,
	 *      org.swixml.Localizer)
	 */
	@SuppressWarnings("unchecked")
	@Override
	public Object convert(final Class type, final Attribute attr,
			final Localizer localizer) throws Exception {
		if (attr != null && attr.getValue() != null) {
			final String[] strs = attr.getValue().split(", *");
			for (int i = 0; i < strs.length; i++) {
				strs[i] = localizer.getString(strs[i]);
			}
			return strs;
		}

		return null;
	}

	/**
	 * Type we ocnvert into (String[].class)
	 * 
	 * @return String[].class
	 * @see org.swixml.Converter#convertsTo()
	 */
	@SuppressWarnings("unchecked")
	@Override
	public Class convertsTo() {
		return String[].class;
	}

}
