package dk.sdu.imada.jkkn04.Protein.gui;

import java.awt.CardLayout;
import java.awt.Color;
import java.awt.Container;
import java.awt.GradientPaint;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.TreeSet;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JFileChooser;
import javax.swing.JList;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.JTextField;
import javax.swing.ListModel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.filechooser.FileFilter;
import javax.swing.table.TableModel;
import javax.swing.text.JTextComponent;

import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.axis.NumberAxis;
import org.jfree.chart.plot.CategoryPlot;
import org.jfree.chart.plot.PiePlot3D;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.renderer.category.BarRenderer;
import org.jfree.data.category.DefaultCategoryDataset;
import org.jfree.data.general.DefaultPieDataset;

import dk.jkkn.util.ResourceFilter;
import dk.jkkn.util.ResourceWrap;
import dk.jkkn.util.Resources;
import dk.sdu.imada.jkkn04.Protein.data.Protein;
import dk.sdu.imada.jkkn04.Protein.data.ProteinMap;
import dk.sdu.imada.jkkn04.Protein.data.ProteinMapCollection;
import dk.sdu.imada.jkkn04.Protein.data.SequenceTranslated;
import dk.sdu.imada.jkkn04.Protein.data.SequenceType;
import dk.sdu.imada.jkkn04.Protein.data.Protein.TranslationType;
import dk.sdu.imada.jkkn04.Protein.tests.AbstractEvaluation;
import dk.sdu.imada.jkkn04.Protein.tests.MCC;
import dk.sdu.imada.jkkn04.Protein.tests.QScore;
import dk.sdu.imada.jkkn04.Protein.tests.SOV;
import dk.sdu.imada.jkkn04.Protein.tests.StateMachine;
import dk.sdu.imada.jkkn04.Protein.tests.StateMachine.ConfusionParameter;
import dk.sdu.imada.jkkn04.Protein.util.LaTeXFormatter;
import dk.sdu.imada.jkkn04.Protein.util.ProcessUtil;
import dk.sdu.imada.jkkn04.Protein.util.ProcessUtil.MultipleEvaluationsMap;
import dk.sdu.imada.jkkn04.Protein.util.ProcessUtil.ObservationPredictionMap;

/**
 * This class implements the logic for the Protein GUI Window.
 * 
 * @author Kristian Krmmer Nielsen <jkkn@jkkn.dk>
 */
public class ProteinWindowImpl extends ProteinWindow {

	private static final NumberFormat DECIMALFORMAT = new DecimalFormat("#.####");

	private ProteinMapCollection proteinCollection = null;
	ObservationPredictionMap analyseResultSum = null;
	boolean disableAutoAnalysis = false;

	private Object previousSelectedObs = null;
	private Object previousSelectedPredict = null;
	private Object previousSelectedObsGraph = null;

	JMenuItem saveMenuItem;

	JList folderList, sequenceList;
	JButton selectFolder, analyseButton, dataModeButton, graphModeButton;

	JPanel modePanel;

	JComboBox selectPredType, selectObsType;

	JTextField sequenceId, predictedType, observedType;
	JTextField detection1, detection2, predicted, observed, primary;

	ResultTable QScoreTable, QScoreAccumTable;
	JTextField QSeqCount, QSeqLength;

	ResultTable MCCTable;
	JTextField MCCSeqCount, MCCSeqLength;

	ResultTable SOVTable;
	JTextField SOVSeqCount, SOVSeqLength;

	ResultTable StateAccTable;
	JTextField SMScore, SMSeqCount, SMSeqLength;
	ResultTable StateTable, StateAccPredTable, StateAccObsTable;

	JTabbedPane graphTabs;
	JPanel graphPanel, piePanel;
	JComboBox selectObsTypeGraph;
	JCheckBox chartByEval;
	
	private static JFreeChart chartResults;
	
	// Folder button action
	private final Action actionSelectFolder = new AbstractAction() {
		private static final long serialVersionUID = 1L;
		private final JFileChooser fileChooser = new JFileChooser();
		{
			fileChooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
		}

		@Override
		public void actionPerformed(final ActionEvent e) {
			if (fileChooser.showOpenDialog(getMainFrame()) == JFileChooser.APPROVE_OPTION) {
				try {
					updateFolderList(fileChooser.getSelectedFile().toURI().toURL());
				} catch (final MalformedURLException e1) {
					e1.printStackTrace();
				}
			}
		}
	};

	// Analyse button action
	private final Action actionAnalyse = new AbstractAction() {
		private static final long serialVersionUID = 1L;

		@Override
		public void actionPerformed(final ActionEvent e) {
			runAnalysis();
		}
	};

	// Graph and data switch button actions
	private final Action actionModeSwitch = new AbstractAction() {
		private static final long serialVersionUID = 1L;

		@Override
		public void actionPerformed(final ActionEvent e) {
			final boolean showData = (e.getSource() == dataModeButton);
			((CardLayout) modePanel.getLayout()).show(modePanel, showData ? "dataMode"
					: "graphMode");
			dataModeButton.setEnabled(!showData);
			graphModeButton.setEnabled(showData);
		}
	};

	// Save as LaTeX button action
	private final Action actionSave = new AbstractAction() {
		private static final long serialVersionUID = 1L;
		private final JFileChooser fileChooser = new JFileChooser();
		{
			fileChooser.setFileFilter(new FileFilter() {
				@Override
				public boolean accept(File f) {
					if (f.isDirectory())
						return true;
					return f.getName().endsWith(".tex");
				}

				@Override
				public String getDescription() {
					return "LaTeX files (*.tex)";
				}
			});
		}

		@Override
		public void actionPerformed(final ActionEvent e) {
			if (fileChooser.showSaveDialog(getMainFrame()) == JFileChooser.APPROVE_OPTION) {
				boolean okay = true;
				if (fileChooser.getSelectedFile().exists()) {
					okay = (JOptionPane.showConfirmDialog(getMainFrame(),
							"File already exists:\n"
									+ fileChooser.getSelectedFile().getName()
									+ "\nAre you sure to overwrite?\n", getMainFrame()
									.getTitle(), JOptionPane.YES_NO_OPTION,
							JOptionPane.WARNING_MESSAGE) == JOptionPane.YES_OPTION);
				}
				if (okay) {
					LaTeXFormatter.writeFile(fileChooser.getSelectedFile(),
							analyseResultSum);
				}
			}
		}
	};


	/**
	 * Creates the main Protein Window implemented version
	 * 
	 * @throws Exception in case of parse errors
	 */
	public ProteinWindowImpl() throws Exception {
		super();
		setupActions();
		/* add graph to graph mode */
		graphPanel.add(createChart());

		/* make more intelligent grid layout for pies */
		piePanel.setLayout(new GridLayout() {
			private static final long serialVersionUID = 1L;
			@Override
			public void layoutContainer(Container parent) {
				
				if (parent.getComponentCount() > 0) {
					// Form layout after space available
					final int cols = Math.max(1, (int) Math.round(Math.sqrt(parent.getComponentCount() / ((double) piePanel.getSize().height / piePanel.getSize().width))));
					final int rows = (int)Math.ceil((double)parent.getComponentCount()/cols);
					setColumns(cols);
					setRows(rows);
				}
				
				super.layoutContainer(parent);
			}
		});
		
		updateContentOfComponents();
		getMainFrame().setVisible(true);
	}

	private void setupActions() {
		saveMenuItem.addActionListener(actionSave);
		selectFolder.addActionListener(actionSelectFolder);
		analyseButton.addActionListener(actionAnalyse);
		dataModeButton.addActionListener(actionModeSwitch);
		graphModeButton.addActionListener(actionModeSwitch);
		folderList.addListSelectionListener(new ListSelectionListener() {
			@Override
			public void valueChanged(final ListSelectionEvent e) {
				updateProteinList();
			}
		});

		sequenceList.addListSelectionListener(new ListSelectionListener() {
			@Override
			public void valueChanged(final ListSelectionEvent e) {
				analyseResultSum = null;
				/*
				 * if only one is selected we will run the analysis automatically
				 */
				if (!disableAutoAnalysis && sequenceList.getSelectedValues().length == 1) {
					runAnalysis();
				} else {
					analyseButton.setEnabled(sequenceList.getSelectedValues().length > 0);
				}
				updateTypeLists();

			}
		});

		final ActionListener updateResults = new ActionListener() {
			@Override
			public void actionPerformed(final ActionEvent e) {
				updateProteinResults();
			}
		};

		selectObsType.addActionListener(updateResults);
		selectPredType.addActionListener(updateResults);
		
		/* graph panel */
		final ActionListener updateChart = new ActionListener() {
			@Override
			public void actionPerformed(final ActionEvent e) {
				updateChart();
			}
		};
		
		selectObsTypeGraph.addActionListener(updateChart);
		chartByEval.addActionListener(updateChart);
		
		graphTabs.addChangeListener(new ChangeListener() {
			@Override
			public void stateChanged(ChangeEvent e) {
				chartByEval.setEnabled(((JTabbedPane)e.getSource()).getSelectedComponent() == graphPanel);
			}
		});

		// Setting default button.
		getMainFrame().getRootPane().setDefaultButton(analyseButton);
	}

	private void updateContentOfComponents() {
		/* update all parts of GUI */
		updateFolderList(null);
		updateProteinList();
		updateTypeLists();
		updateProteinResults();
	}

	/**
	 * This updates the list of proteins sequences
	 */
	protected void updateProteinList() {
		try {
			if (folderList.getSelectedIndex() != -1) {
				proteinCollection = new ProteinMapCollection(((URLView) folderList
						.getSelectedValue()).url);
				analyseResultSum = null;
				sequenceList.setListData(proteinCollection.getProteinIdentifiers()
						.toArray());

				// Select all by default
				final int[] selected = new int[sequenceList.getModel().getSize()];
				for (int i = 0; i < selected.length; i++) {
					selected[i] = i;
				}
				if (selected.length > 1) {
					disableAutoAnalysis = true;
				}
				sequenceList.setSelectedIndices(selected);
				disableAutoAnalysis = false;
			} else {
				sequenceList.setListData(new Object[] {});
			}
		} catch (final IOException e) {
			// TODO: Display warning
			e.printStackTrace();
		}
	}

	/**
	 * Updates list of available folders
	 * 
	 * @param otherDirectory Custom selected folder to also add.
	 */
	protected void updateFolderList(final URL otherDirectory) {
		final List<URLView> folders = new ArrayList<URLView>();
		final URLView otherDir = otherDirectory != null ? new URLView(otherDirectory)
				: null;

		try {
			for (final URL file : Resources.getDirectoryList(
					ProteinMapCollection.DEFAULT_DATA_PATH, new ResourceFilter() {

						@Override
						public boolean accept(final ResourceWrap resource) {
							return resource.isDirectory();
						}

					})) {
				folders.add(new URLView(file));
			}
		} catch (final IOException e) {
			// FIXME: Throw something
			e.printStackTrace();
		}

		if (otherDir != null) {
			folders.add(otherDir);
		}

		folderList.setListData(folders.toArray());

		if (otherDir != null) {
			folderList.setSelectedValue(otherDir, true);
		}

	}

	/**
	 * Updates comboboxes for prediction and observed types
	 */
	void updateTypeLists() {
		/* try to preserve selected items */
		if (selectObsType.getSelectedIndex() != -1) {
			previousSelectedObs = selectObsType.getSelectedItem();
		}
		if (selectPredType.getSelectedIndex() != -1) {
			previousSelectedPredict = selectPredType.getSelectedItem();
		}
		if (selectObsTypeGraph.getSelectedIndex() != -1) {
			previousSelectedObsGraph = selectObsTypeGraph.getSelectedItem();
		}

		selectObsType.removeAllItems();
		selectPredType.removeAllItems();
		selectObsTypeGraph.removeAllItems();
		if (proteinCollection != null && analyseResultSum != null) {
			final Set<SequenceTranslated> allTypes = new TreeSet<SequenceTranslated>();
			/*
			 * make map of available types looking at which protein that are selected
			 */
			for (final Object sequence : sequenceList.getSelectedValues()) {
				allTypes.addAll(proteinCollection.getProteinMap((String) sequence)
						.keySet());
			}

			for (final SequenceTranslated st : allTypes) {
				if (st.getSequenceType().isObservation()) {
					selectObsType.addItem(st);
					selectObsTypeGraph.addItem(st);
				}
				if (st.getSequenceType().isPrediction()) {
					selectPredType.addItem(st);
				}
			}

			/* preserve selection */
			if (previousSelectedObs != null
					&& previousSelectedObs instanceof SequenceTranslated) {
				/*
				 * ignored if no longer in combobox
				 */
				selectObsType.setSelectedItem(previousSelectedObs);
			}
			if (previousSelectedPredict != null
					&& previousSelectedPredict instanceof SequenceTranslated) {
				/*
				 * ignored if no longer in combobox
				 */
				selectPredType.setSelectedItem(previousSelectedPredict);
			}
			if (previousSelectedObsGraph != null
					&& previousSelectedObsGraph instanceof SequenceTranslated) {
				/*
				 * ignored if no longer in combobox
				 */
				selectObsTypeGraph.setSelectedItem(previousSelectedObsGraph);
			}
		}
	}

	/**
	 * This fills in all the result fields in the window.
	 */
	void updateProteinResults() {
		/* reset all fields first */
		for (final JTextComponent field : new JTextComponent[] { sequenceId,
				predictedType, observedType, detection1, detection2, predicted, observed,
				primary, QSeqCount, QSeqLength, MCCSeqCount, MCCSeqLength, SMScore,
				SOVSeqCount, SOVSeqLength, SMSeqCount, SMSeqLength, }) {
			field.setText("");
		}

		/* reset all table values */
		for (final ResultTable table : new ResultTable[] { QScoreTable, QScoreAccumTable,
				MCCTable, SOVTable, StateAccTable, StateTable, StateAccPredTable,
				StateAccObsTable }) {
			final TableModel model = table.getModel();
			for (int c = 0; c < model.getColumnCount(); c++) {
				for (int r = 0; r < model.getRowCount(); r++) {
					model.setValueAt(null, r, c);
				}
			}
		}

		if (proteinCollection != null && analyseResultSum != null
				&& !analyseResultSum.isEmpty() && selectObsType.getSelectedIndex() != -1
				&& selectPredType.getSelectedIndex() != -1) {

			final SequenceTranslated selectedObs = (SequenceTranslated) selectObsType
					.getSelectedItem();
			final SequenceTranslated selectedPred = (SequenceTranslated) selectPredType
					.getSelectedItem();

			final Object proteins[] = sequenceList.getSelectedValues();

			if (proteins.length != 1) {
				sequenceId.setText(proteins.length > 1 ? "[multiple selected]" : "");
			} else {
				final String id = (String) proteins[0];
				final ProteinMap proteinMap = proteinCollection.getProteinMap(id);
				final Protein obsProtein = proteinMap.get(selectedObs);
				final Protein predProtein = proteinMap.get(selectedPred);
				sequenceId.setText(id);
				predictedType.setText(predProtein.getDescription());
				observedType.setText(obsProtein.getDescription());
				predicted.setText(predProtein.getCleanSequence());
				observed.setText(obsProtein.getCleanSequence());

				/* Try to find primary sequence */
				final Protein primaryProtein = proteinMap.get(new SequenceTranslated(
						SequenceType.PRIMARY, TranslationType.RAW));
				if (primaryProtein != null) {
					primary.setText(primaryProtein.getSequence());
				}
			}

			/* Fill in all the various tables */
			final MultipleEvaluationsMap resultsPerAnalysis = analyseResultSum.get(
					selectedObs).get(selectedPred);
			for (final AbstractEvaluation e : resultsPerAnalysis.values()) {
				final Map<String, Object> results = e.getResults();
				if (e instanceof QScore) {
					TableModel model = QScoreTable.getModel();
					ListModel rowModel = QScoreTable.getRowModel();
					for (int c = 0; c < model.getColumnCount(); c++) {
						final String colName = model.getColumnName(c);
						for (int r = 0; r < model.getRowCount(); r++) {
							final String rowName = (String) rowModel.getElementAt(r);
							model.setValueAt(results.get("Q-" + rowName + colName), r, c);
						}
					}

					model = QScoreAccumTable.getModel();
					rowModel = QScoreAccumTable.getRowModel();
					for (int r = 0; r < model.getRowCount(); r++) {
						final String rowName = (String) rowModel.getElementAt(r);
						model.setValueAt(results.get(rowName), r, 0);
					}

					QSeqCount.setText(Integer
							.toString((Integer) results.get("sequences")));
					QSeqLength.setText(Integer.toString((Integer) results.get("length")));

				} else if (e instanceof MCC) {

					final TableModel model = MCCTable.getModel();
					final ListModel rowModel = MCCTable.getRowModel();
					for (int r = 0; r < model.getRowCount(); r++) {
						final String rowName = (String) rowModel.getElementAt(r);
						model.setValueAt(results.get(rowName), r, 0);
					}

					MCCSeqCount.setText(Integer.toString((Integer) results
							.get("sequences")));
					MCCSeqLength.setText(Integer
							.toString((Integer) results.get("length")));

				} else if (e instanceof SOV) {

					final TableModel model = SOVTable.getModel();
					final ListModel rowModel = SOVTable.getRowModel();
					for (int r = 0; r < model.getRowCount(); r++) {
						final String rowName = (String) rowModel.getElementAt(r);
						model.setValueAt(results.get(rowName), r, 0);
					}

					SOVSeqCount.setText(Integer.toString((Integer) results
							.get("sequences")));
					SOVSeqLength.setText(Integer
							.toString((Integer) results.get("length")));

				} else if (e instanceof StateMachine) {

					SMScore.setText(DECIMALFORMAT.format(results.get("State")));

					/* fill in all sum tables */
					final Map<ResultTable, String> tables = new HashMap<ResultTable, String>();
					tables.put(StateAccTable, "");
					tables.put(StateAccPredTable, "-Predicted");
					tables.put(StateAccObsTable, "-Observed");
					for (final Map.Entry<ResultTable, String> tableToPrefix : tables
							.entrySet()) {

						final ResultTable table = tableToPrefix.getKey();
						final String prefix = tableToPrefix.getValue();

						final TableModel model = table.getModel();
						final ListModel rowModel = table.getRowModel();
						for (int c = 0; c < model.getColumnCount(); c++) {
							final String colName = model.getColumnName(c);
							for (int r = 0; r < model.getRowCount(); r++) {
								String rowName = ((String) rowModel.getElementAt(r))
										.toUpperCase();

								/*
								 * construct logical name from row name (e.g. Bridges =>
								 * BRIDGE)
								 */
								if (rowName.endsWith("S")) {
									rowName = rowName.substring(0, rowName.length() - 1);
								}

								String key;
								if (colName.equals("Total")) {
									key = "State-Sum-" + rowName;
								} else {
									key = "State-Sum-" + colName + prefix + "-" + rowName;
								}

								model.setValueAt(results.get(key), r, c);

							}
						}
					}

					/*
					 * Detailed table as (Bridges/Gaps/Expansions/Reductions/Other)
					 */
					final ConfusionParameter order[] = new ConfusionParameter[] {
							ConfusionParameter.BRIDGE, ConfusionParameter.GAP,
							ConfusionParameter.EXPANSION, ConfusionParameter.REDUCTION,
							ConfusionParameter.OTHER };
					final TableModel model = StateTable.getModel();
					final ListModel rowModel = StateTable.getRowModel();
					for (int c = 0; c < model.getColumnCount(); c++) {
						final String colName = model.getColumnName(c);
						for (int r = 0; r < model.getRowCount(); r++) {

							final String rowName = (String) rowModel.getElementAt(r);
							final String baseKey = "State-" + rowName + colName + "-";

							/* construct entry (b/g/e/r/o) */
							final StringBuffer entry = new StringBuffer();
							for (int i = 0; i < order.length; i++) {
								if (i > 0) {
									entry.append(" / ");
								}
								entry.append(results.get(baseKey + order[i].name()));
							}

							/* ignore diagonal fields */
							if (r == c && entry.toString().equals("0 / 0 / 0 / 0 / 0")) {
								model.setValueAt("-", r, c);
							} else {
								model.setValueAt(entry.toString(), r, c);
							}
						}
					}

					SMSeqCount.setText(Integer.toString((Integer) results
							.get("sequences")));
					SMSeqLength
							.setText(Integer.toString((Integer) results.get("length")));

					/* generate sequence type strings */
					final String[] mismatchBySeq = ((StateMachine) e)
							.getMismatchesBySequence();
					detection1.setText(mismatchBySeq[0]);
					detection2.setText(mismatchBySeq[1]);

				}
			}

			/*
			 * Place all carets at zero (so first parts of strings are shown/not last)
			 */
			for (final JTextComponent field : new JTextComponent[] { sequenceId,
					predictedType, observedType, predicted, observed, detection1,
					detection2, primary }) {
				field.setCaretPosition(0);
			}

			/* Enable save action */
			saveMenuItem.setEnabled(true);
		} else {
			/* Empty: disable save action */
			saveMenuItem.setEnabled(false);
		}
		
	}

	/**
	 * This runs the analysis by comparing all combinations of observed types and
	 * predicted types for the selected sequences
	 */
	public void runAnalysis() {
		analyseButton.setEnabled(false);

		if (proteinCollection != null && sequenceList.getSelectedValues().length > 0) {

			analyseResultSum = new ObservationPredictionMap();

			for (final Object sequence : sequenceList.getSelectedValues()) {

				final ProteinMap pm = proteinCollection.getProteinMap((String) sequence);
				final ObservationPredictionMap results = ProcessUtil
						.performAllTestsByType(pm, SequenceType.OBSERVATION_TYPES,
								SequenceType.PREDICTION_TYPES);

				ProcessUtil.summerizeAll(analyseResultSum, results);
			}

			updateTypeLists();
		} else {
			JOptionPane.showMessageDialog(getMainFrame(),
					"Select directory and proteins first.", "Notice",
					JOptionPane.ERROR_MESSAGE);
		}
	}

	/**
	 * Creates a new bar chart view of the differences
	 * 
	 * @return Chart
	 */
	private static ChartPanel createChart() {
		chartResults = ChartFactory.createBarChart(null,
				null, "Score (0-100)", null, PlotOrientation.VERTICAL, true,
				true, false);
		final CategoryPlot plot = (CategoryPlot) chartResults.getPlot();
		plot.setBackgroundPaint(Color.LIGHT_GRAY);
		plot.setDomainGridlinePaint(Color.WHITE);
		plot.setDomainGridlinesVisible(true);
		plot.setRangeGridlinePaint(Color.WHITE);
		NumberAxis numberAxis = (NumberAxis) plot.getRangeAxis();
		numberAxis.setAutoRangeIncludesZero(false);
		BarRenderer barRenderer = (BarRenderer) plot.getRenderer();
		barRenderer.setDrawBarOutline(true);
		barRenderer.setIncludeBaseInRange(false);
		/* default colours / does not hold if grouping by evaluators */
		GradientPaint mcc = new GradientPaint(0, 0, Color.GREEN, 0, 0,
				new Color(0, 64, 0));
		GradientPaint q3 = new GradientPaint(0, 0, Color.BLUE, 0, 0,
				new Color(0, 0, 64));
		GradientPaint sov3 = new GradientPaint(0, 0, Color.RED, 0, 0,
				new Color(64, 0, 0));
		GradientPaint smcm = new GradientPaint(0, 0, Color.ORANGE, 0, 0,
				new Color(64, 64, 0));
		barRenderer.setSeriesPaint(0, mcc);
		barRenderer.setSeriesPaint(1, q3);
		barRenderer.setSeriesPaint(2, sov3);
		barRenderer.setSeriesPaint(3, smcm);
		return new ChartPanel(chartResults, true);
	}

	/**
	 * Updated results for making up graph
	 */
	protected void updateChart() {
		final CategoryPlot plot = (CategoryPlot) chartResults.getPlot();

		/* Split Pie plot into the different predictors */
		piePanel.removeAll();
	
		if (proteinCollection != null && analyseResultSum != null
				&& !analyseResultSum.isEmpty() && selectObsTypeGraph.getSelectedIndex() != -1) {
			final SequenceTranslated obs = (SequenceTranslated) selectObsTypeGraph
			.getSelectedItem();
			
			
			final Map<SequenceTranslated, MultipleEvaluationsMap> data = analyseResultSum.get(
					obs);
			
			/* Update Bar plot and Pie plot */
			final DefaultCategoryDataset ds = new DefaultCategoryDataset();
			final boolean byEval = chartByEval.isSelected();
			
			chartResults.setTitle(obs.toString());
			
			for (SequenceTranslated prediction: new TreeSet<SequenceTranslated>(data.keySet())) {
				
				for (AbstractEvaluation eval: data.get(prediction).values()) {
					final Map<String, Object> results = eval.getResults();
					Double value = null;
					String type = null;
					
					if (eval instanceof MCC) {
						double stacked = ((Double) results.get("MCC-H") + (Double) results.get("MCC-E") + (Double) results.get("MCC-C")) / 3;
						value = (stacked + 1) / 0.02; // calculate percentage
						type = "MCC";
					} else if (eval instanceof QScore) {
						value = (Double) results.get("Q3");
						type = "Q3";
					} else if (eval instanceof SOV) {
						value = (Double)results.get("SOV3");
						type = "SOV3";
					} else if (eval instanceof StateMachine) {
						value = (Double)results.get("State");
						type = "SMCM";
					}
					
					if (value != null) {
						if (byEval) {
							ds.addValue(value, prediction, type);
						} else {
							ds.addValue(value, type, prediction);
						}
					}
					
				}
				
				StateMachine sm = (StateMachine) data.get(prediction).get(StateMachine.class.getName());
				
				if (sm != null) {
					final Map<String, Object> results = sm.getResults();
					/* Create pie charts for SMCM */
					final DefaultPieDataset pieData = new DefaultPieDataset();
					
					for (String category: new String[] {"Bridges", "Gaps", "Expansions", "Reductions", "Other"}) {
						
						String catName = category.toUpperCase();
						if (catName.endsWith("S")) {
							catName = catName.substring(0, category.length()-1);
						}
						pieData.setValue(category, (Integer) results.get("State-Sum-" + catName));
					}
					
			        final JFreeChart pieChart = ChartFactory.createPieChart3D(prediction.toString(), pieData, false, true, false);
			        final PiePlot3D pie = (PiePlot3D)pieChart.getPlot();
			        pie.setForegroundAlpha(0.6F);
			        pie.setCircular(true);
			        piePanel.add(new ChartPanel(pieChart, true));
				}
				
			}
			plot.setDataset(ds);
			plot.getDomainAxis().setLabel(byEval ? "Evaluator" : "Predictor");
		} else {
			plot.setDataset(null);
			chartResults.setTitle((String)null);
			plot.getDomainAxis().setLabel(null);
		}
		piePanel.validate();
		piePanel.repaint(); // workaround needed
	}

	/**
	 * Private class used to display the folder name in the folder list Only displays the
	 * last folder entry.
	 */
	private static class URLView {
		public URL url;
		private String file;

		public URLView(final URL url) {
			this.url = url;
			file = url.getFile();

			// cut off path unless at end of path
			final int lastSlash = file.lastIndexOf("/", file.length() - 2);
			if (lastSlash != -1) {
				file = file.substring(lastSlash + 1);
			}
		}

		@Override
		public String toString() {
			return file;
		}
	}

}
