/**
 * 
 */
package dk.sdu.imada.jkkn04.Protein.gui;

import java.awt.Component;
import java.util.Arrays;

import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.ListCellRenderer;
import javax.swing.ListModel;
import javax.swing.ListSelectionModel;
import javax.swing.SwingConstants;
import javax.swing.UIManager;
import javax.swing.border.CompoundBorder;
import javax.swing.border.EmptyBorder;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.JTableHeader;
import javax.swing.table.TableModel;

/**
 * @author Kristian Krmmer Nielsen <jkkn@jkkn.dk>
 */
public class ResultTable extends JScrollPane {

	private static final long serialVersionUID = 1L;

	/**
	 * Default type used inside table fields (determinds alignment)
	 */
	Class<?> defaultType = String.class;
	private final JTable table;
	private Object defaultValue = "";
	private String[] headers = new String[0];
	private String[] rowHeaders = new String[0];

	/**
	 * Construct a result table, which also contains row header.
	 */
	public ResultTable() {
		super();

		table = new JTable();
		updateModel();

		setViewportView(table);
		table.setAutoResizeMode(JTable.AUTO_RESIZE_ALL_COLUMNS);
		table.setCellSelectionEnabled(true);
		table.getColumnModel().getSelectionModel().setSelectionMode(
				ListSelectionModel.SINGLE_INTERVAL_SELECTION);
		((DefaultTableCellRenderer) table.getTableHeader().getDefaultRenderer())
				.setHorizontalAlignment(SwingConstants.RIGHT);
		// setPreferredSize(new Dimension(200,200));

	}

	/**
	 * Class for generating the row header for the table
	 */
	private class RowHeaderRenderer extends JLabel implements ListCellRenderer {
		private static final long serialVersionUID = 1L;

		RowHeaderRenderer(final JTable table) {
			final JTableHeader header = table.getTableHeader();
			setOpaque(true);
			setBorder(new CompoundBorder(UIManager.getBorder("TableHeader.cellBorder"),
					new EmptyBorder(0, 20, 0, 20)));
			setHorizontalAlignment(CENTER);
			setForeground(header.getForeground());
			setBackground(header.getBackground());
			setFont(header.getFont());

		}

		public Component getListCellRendererComponent(final JList list,
				final Object value, final int index, final boolean isSelected,
				final boolean cellHasFocus) {
			setText((value == null) ? "" : value.toString());
			return this;
		}
	}

	/**
	 * Change default value put into the table cells
	 * 
	 * @param value new value
	 */
	public void setDefaultValue(final Object value) {
		defaultValue = value;
		updateModel();
	}

	/**
	 * Change default type for the table cells
	 * 
	 * @param type new class type
	 */
	public void setDefaultType(final Class<?> type) {
		defaultType = type;
		updateModel();
	}

	/**
	 * Sets the column headers for the table
	 * 
	 * @param headers column headers
	 */
	public void setHeaders(final String[] headers) {
		this.headers = headers;
		updateModel();
	}

	/**
	 * Set the row headers for the table
	 * 
	 * @param rowHeaders row headers
	 */
	public void setRowHeaders(final String[] rowHeaders) {
		this.rowHeaders = rowHeaders;
		updateModel();
	}

	/**
	 * Creates an updated model for the table
	 */
	private void updateModel() {

		// Cast default value
		if (!defaultType.isInstance(defaultValue)) {
			try {
				defaultValue = defaultType.getConstructor(defaultValue.getClass())
						.newInstance(defaultValue);
			} catch (final Exception e) {
				e.printStackTrace();
				System.exit(1);
			}
		}

		final Object[][] defaults = new Object[rowHeaders.length][headers.length];

		for (int r = 0; r < rowHeaders.length; r++) {
			Arrays.fill(defaults[r], defaultValue);
		}

		table.setModel(new DefaultTableModel(defaults, headers) {
			private static final long serialVersionUID = 1L;
			private final Class<?> type = defaultType;

			@Override
			public Class<?> getColumnClass(final int columnIndex) {
				return type;
			}

			@Override
			public boolean isCellEditable(final int rowIndex, final int columnIndex) {
				return false;
			}
		});

		// row header
		final JList rowHeader = new JList(rowHeaders);
		rowHeader.setFixedCellHeight(table.getRowHeight());
		rowHeader.setCellRenderer(new RowHeaderRenderer(table));
		rowHeader.setOpaque(false);
		setRowHeaderView(rowHeader);

	}

	/**
	 * Return the currently active model
	 * 
	 * @return table model
	 */
	public TableModel getModel() {
		return table.getModel();
	}

	/**
	 * Returns the list model for the row header of the table
	 * 
	 * @return list model
	 */
	public ListModel getRowModel() {
		return ((JList) getRowHeader().getView()).getModel();
	}

}
