/**
 * 
 */
package dk.sdu.imada.jkkn04.Protein.util;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Locale;
import java.util.Map;
import java.util.TreeSet;

import dk.sdu.imada.jkkn04.Protein.data.SequenceTranslated;
import dk.sdu.imada.jkkn04.Protein.tests.MCC;
import dk.sdu.imada.jkkn04.Protein.tests.QScore;
import dk.sdu.imada.jkkn04.Protein.tests.SOV;
import dk.sdu.imada.jkkn04.Protein.tests.StateMachine;
import dk.sdu.imada.jkkn04.Protein.util.ProcessUtil.MultipleEvaluationsMap;
import dk.sdu.imada.jkkn04.Protein.util.ProcessUtil.ObservationPredictionMap;

/**
 * Small class to output a LaTeX file of tables of interesting results
 * 
 * @author Kristian Krmmer Nielsen <jkkn@jkkn.dk>
 */
public class LaTeXFormatter {
	
	static {
		Locale.setDefault(Locale.ENGLISH);
	}

	/**
	 * Writes a set of results to a LaTeX formatted file
	 * 
	 * @param file File to write into
	 * @param results Results to write
	 */
	public static void writeFile(File file, ObservationPredictionMap results) {

		File out = file;

		/* Add ".tex" extension if missing */
		if (file.getName().indexOf('.') == -1) {
			out = new File(file.getAbsoluteFile() + ".tex");
		}

		PrintWriter pw = null;
		try {
			pw = new PrintWriter(new BufferedWriter(new FileWriter(out)));

			pw.println("%% Created by Protein Analyser (http://jkkn.dk/sdu/proteins/)");
			pw.println();
			
			/* For each observation compare to each prediction */
			for (final SequenceTranslated observedType: new TreeSet<SequenceTranslated>(results.keySet())) {
				final Map<SequenceTranslated, MultipleEvaluationsMap> mapByObserved = results.get(observedType);
				for (final SequenceTranslated predictionType: new TreeSet<SequenceTranslated>(mapByObserved.keySet())) {
					final MultipleEvaluationsMap resultsByAnalyse = mapByObserved.get(predictionType);
					final Map<String, Object> sm  = resultsByAnalyse.get(StateMachine.class.getName()).getResults();
					final Map<String, Object> mcc = resultsByAnalyse.get(MCC.class.getName()).getResults();
					final Map<String, Object> sov = resultsByAnalyse.get(SOV.class.getName()).getResults();
					final Map<String, Object> q   = resultsByAnalyse.get(QScore.class.getName()).getResults();
					
					pw.format("\\subsubsection*{%s compared to %s}",
							observedType.toString().replace("_", "\\_"),
							predictionType.toString().replace("_", "\\_"));
					pw.println();
					pw.println();
					pw.println("\\begin{table}[H]");
					pw.println("\\begin{centering}");
					pw.println("\\begin{tabular}{r|r|r|r|r|}");
					pw.println("& \\textbf{H} & \\textbf{E} & \\textbf{C} & \\textbf{Total/Score}\\\\");
					pw.println("\\hline");
					
					for (String category: new String[] {"Bridges", "Gaps", "Expansions", "Reductions"}) {
						
						String cateName = category.toUpperCase().substring(0, category.length()-1); // cut off 's'
						pw.format("\\textbf{%s} & %,d & %,d & %,d & %,d\\\\",
								category,
								sm.get("State-Sum-H-" + cateName),
								sm.get("State-Sum-E-" + cateName),
								sm.get("State-Sum-C-" + cateName),
								sm.get("State-Sum-" + cateName)
								);
						pw.println();
					}
					pw.format("\\textbf{Other} & - & - & - & %,d\\\\",
							sm.get("State-Sum-OTHER"));
					pw.println();
					
					
					pw.format("\\textbf{MCC} & %,.3f & %,.3f & %,.3f & -\\\\",
							mcc.get("MCC-H"),
							mcc.get("MCC-E"),
							mcc.get("MCC-C")
							);
					pw.println();

					pw.format("\\textbf{SOV} & %,.3f & %,.3f & %,.3f & %,.3f\\\\",
							sov.get("SOV-H"),
							sov.get("SOV-E"),
							sov.get("SOV-C"),
							sov.get("SOV3"));
					pw.println();
				
					pw.format("\\textbf{Q Score} & %,.3f & %,.3f & %,.3f & %,.3f\\\\",
							q.get("Q-H"),
							q.get("Q-E"),
							q.get("Q-C"),
							q.get("Q3"));
					pw.println();

					pw.println("\\hline");
					pw.println("\\end{tabular}");
					
					pw.println("\\par\\end{centering}");
					pw.format("\\caption{$Q_{3}=%,.3f$, $SOV_{3}=%,.3f$, $SMCM=%,.3f\\%%$.}",
							q.get("Q3"),
							sov.get("SOV3"),
							sm.get("State")
							);
					pw.println();
					pw.println("\\end{table}");
					pw.println();
					pw.println();
					
				}
			}

		} catch (IOException e) {
			// FIXME: Throw something
			e.printStackTrace();
		} finally {
			if (pw != null) pw.close();
		}

	}

}
